(**************************************************************************)

open Format

(*s Local refs to set the options *)

let verbose_ = ref false
let debug_ = ref false
let show_time_ = ref false
let old_ = ref false
let postck_ = ref true
let invgen_ = ref false
let domain_ = ref false

type CrefDomain = Sll | Shapes

(*s Parsing the command-line *)

let lcopying () =
  eprintf "\
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License version 2, as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License version 2 for more details
(enclosed in the file GPL).
";
  flush stderr

let lbanner () =  eprintf "\
This is why version %s, compiled on %s
Copyright (c) 2009 LIAFA
This is free software with ABSOLUTELY NO WARRANTY (use option -warranty)
" Cref_version.version Cref_version.date;
  flush stderr

let lusage () =
  eprintf "\
Usage: whyl [options] [files...]

If no file is given, the source is read on standard input and
output is written in file `out...'

Generic Options:
  -h, --help     prints this message
  -V, --verbose  verbose mode
  -q, --quiet    quiet mode (default)
  -d, --debug    debugging mode (implies verbose mode)
  -v, --version  prints version and exits
  -w, --why      call why
  --show-time    prints execution time

Analysis Options:
  -post, --post-check    post-condition reasoning
  -inv, --inv-gen        invariant generation
  -shp, --shapes         using abstract domain of shapes
			 (default)
  -sll, --singly-ll      using abstract domain of singly
                         linked lists
";
  flush stderr

let files =
  let filesq = ref [] in
  let rec parse = function
    | [] -> List.rev !filesq
    | ("-h" | "-help" | "--help") :: _ -> lusage (); exit 0
    | ("-d" | "--debug") :: args -> verbose_ := true; debug_ := true; parse args
    | ("-q" | "--quiet") :: args -> verbose_ := false; parse args
    | ("-v" | "-version" | "--version") :: _ -> lbanner (); exit 0
    | ("-V" | "--verbose") :: args -> verbose_ := true; parse args
    | ("-w" | "--why") :: args -> old_ := true; parse args
    | ("-show-time" | "--show-time") :: args -> show_time_ := true; parse args
    | ("-post" | "--post-check") :: args -> postck_ := true; parse args
    | ("-inv"  | "--inv-gen") :: args -> invgen_ := true; parse args
    | ("-shp"  | "--shapes") :: args -> domain_ := Shapes; parse args
    | ("-sll"  | "--singly-ll") :: args -> domain_ := Sll; parse args
    | f :: args ->
        filesq := f :: !filesq; parse args
  in
  parse (List.tl (Array.to_list Sys.argv))

(*s Finally, we dereference all the refs *)

let verbose:bool = !verbose_
let debug:bool = !debug_
let show_time:bool = !show_time_
let old:bool = !old_
let postck:bool = !postck_
let invgen:bool = !invgen_
let domain:CrefDomain = !domain_


let if_verbose f x = if verbose then f x
let if_verbose_2 f x y = if verbose then f x y
let if_verbose_3 f x y z = if verbose then f x y z

let if_debug f x = if debug then f x
let if_debug_2 f x y = if debug then f x y
let if_debug_3 f x y z = if debug then f x y z

